@@ -4,10 +4,14 @@ from bokeh.models import *
 from bokeh.plotting import Figure
 from openautomatumdronedata.dataset import droneDataset
 from bokeh.palettes import Dark2_5 as palette
+from bokeh.palettes import Category20_20 as palette_major
 import os
 import itertools  
 import math
 import copy
+import numpy as np 
+
+
 
 def generate_lines_to_plot_object(x, y, psi, l = 5, w = 2):
     """
@@ -37,6 +41,21 @@ def generate_lines_to_plot_object(x, y, psi, l = 5, w = 2):
     return(x_vec, y_vec)
 
 
+def generate_lines_to_lane_marking(x, y, psi, line_length, direction=1.0):
+    """
+    This function generates the lines to visualize the orthogonal distance between vehicle center point and line
+    """
+
+    psi_ort = psi + (direction * np.pi / 2)
+
+
+    x_le = math.cos(psi_ort) * line_length + x
+    y_le = math.sin(psi_ort) * line_length + y
+
+    return [x, x_le], [y, y_le]
+
+
+
 class automatumDataBokeh():
     """
         Main Class for the automatum bokeh server
@@ -46,6 +65,8 @@ class automatumDataBokeh():
         self.curDoc = curDoc
         self._reset_internal_states()
 
+
+
         self.curDoc.add_root(self.load_bokeh_objects())
 
     def _reset_internal_states(self):
@@ -56,7 +77,8 @@ class automatumDataBokeh():
         self.live_plot_time = 0
         self.dataset = None
         self.live_video_play = False
-        self.static_world_line_dict = None        
+        self.static_world_line_dict = None      
+        self.uuid_color_mapping_dict = None   
     #
     #
     # Rendering
@@ -70,13 +92,15 @@ class automatumDataBokeh():
         self.static_world_line_dict = {
             "xdata": [],
             "ydata": [],
-            "color": [],            
+            "color": [],  
+            "info": []          
         }
 
         static_line_list = self.dataset.statWorld.get_lane_marking_dicts()
         for lane_markings in static_line_list:
             self.static_world_line_dict["xdata"].append(lane_markings["x_vec"])
             self.static_world_line_dict["ydata"].append(lane_markings["y_vec"])
+            self.static_world_line_dict["info"].append("Type: %s" % lane_markings["type"])
             if(lane_markings["type"] == "broken"):
                 self.static_world_line_dict["color"].append("grey")
             else:
@@ -99,26 +123,55 @@ class automatumDataBokeh():
         colors = itertools.cycle(palette)
         
         objList = self.dataset.dynWorld.get_list_of_dynamic_objects_for_specific_time(time)
-        for color, obj in zip(colors, objList):
+        for color_unique, obj in zip(colors, objList):
             idx = obj.next_index_of_specific_time(time)
             (x_vec, y_vec) = generate_lines_to_plot_object(obj.x_vec[idx], obj.y_vec[idx], obj.psi_vec[idx], l = obj.length, w = obj.width)
             live_plot_dict["xdata"].append(x_vec)
             live_plot_dict["ydata"].append(y_vec)
-            live_plot_dict["color"].append(color)
-            # relation_dict = obj.object_relation_dict_list[idx]
-            # for key, value in relation_dict.items():
-            #     if(value is not None and "front" in key):
-            #         relation_objet = self.dataset.dynWorld.get_dynObj_by_UUID(value)
-            #         idx_relation_object = relation_objet.next_index_of_specific_time(time)
-            #         live_plot_dict["xdata"].append([obj.x_vec[idx], relation_objet.x_vec[idx_relation_object]])
-            #         live_plot_dict["ydata"].append([obj.y_vec[idx], relation_objet.y_vec[idx_relation_object]])
-            #         if("left" in key):
-            #             live_plot_dict["color"].append("blue")   
-            #         if("right" in key):
-            #             live_plot_dict["color"].append("green")       
-            #         if("ego" in key):
-            #             live_plot_dict["color"].append("red")                                                          
+            live_plot_dict["color"].append(self.uuid_color_mapping_dict[obj.UUID])
+            if(obj.tth_dict_vec is not None):
+                if(obj.tth_dict_vec[idx]["front_ego"] is None or obj.tth_dict_vec[idx]["front_ego"] < 0):
+                    tth = "-"
+                else:
+                    tth = "%.3f" % obj.tth_dict_vec[idx]["front_ego"]
+            if(obj.ttc_dict_vec is not None):
+                if(obj.ttc_dict_vec[idx]["front_ego"] is None or obj.ttc_dict_vec[idx]["front_ego"] < 0):
+                    ttc = "-"
+                else:
+                    ttc = "%.3f" % obj.ttc_dict_vec[idx]["front_ego"]
 
+
+            live_plot_dict["info"].append("UUID: %s - v_x: %.3f - v_y: %.3f - TTC Front: %s - TTH Front: %s" % (obj.UUID, obj.vx_vec[idx], obj.vy_vec[idx], ttc, tth))
+
+            if(0 in self.extra_plot_option_checkbox.active and obj.object_relation_dict_list is not None):
+                relation_dict = obj.object_relation_dict_list[idx]
+                for key, value in relation_dict.items():
+                    if(value is not None and "front" in key):
+                        relation_objet = self.dataset.dynWorld.get_dynObj_by_UUID(value)
+                        idx_relation_object = relation_objet.next_index_of_specific_time(time)
+                        live_plot_dict["xdata"].append([obj.x_vec[idx], relation_objet.x_vec[idx_relation_object]])
+                        live_plot_dict["ydata"].append([obj.y_vec[idx], relation_objet.y_vec[idx_relation_object]])
+                        if("left" in key):
+                            live_plot_dict["color"].append("blue")   
+                        if("right" in key):
+                            live_plot_dict["color"].append("green")       
+                        if("ego" in key):
+                            live_plot_dict["color"].append("red")   
+                        live_plot_dict["info"].append("%s from %s to %s" % (key, obj.UUID, value))
+
+            if(1 in self.extra_plot_option_checkbox.active and obj.distance_left_lane_marking is not None and obj.distance_right_lane_marking is not None):
+                if(obj.distance_left_lane_marking[idx] >= 0):
+                    x_vec, y_vec = generate_lines_to_lane_marking(obj.x_vec[idx], obj.y_vec[idx], obj.psi_vec[idx], obj.distance_left_lane_marking[idx], direction=1.0)                                                 
+                    live_plot_dict["xdata"].append(x_vec)
+                    live_plot_dict["ydata"].append(y_vec)
+                    live_plot_dict["color"].append("blue")
+                    live_plot_dict["info"].append("DistLeft: %.3f" % obj.distance_left_lane_marking[idx])
+                if(obj.distance_right_lane_marking[idx] >= 0):
+                    x_vec, y_vec = generate_lines_to_lane_marking(obj.x_vec[idx], obj.y_vec[idx], obj.psi_vec[idx], obj.distance_right_lane_marking[idx], direction=-1.0)                                                 
+                    live_plot_dict["xdata"].append(x_vec)
+                    live_plot_dict["ydata"].append(y_vec)
+                    live_plot_dict["color"].append("green")        
+                    live_plot_dict["info"].append("DistRight: %.3f" % obj.distance_right_lane_marking[idx])        
         self.live_plot_data.data = live_plot_dict
 
         self.livePlotTitle.text = 'Live plot of dataset (t = %.2f/%.2f):'%(time, self.dataset.dynWorld.get_length_of_dataset_in_seconds())
@@ -129,7 +182,8 @@ class automatumDataBokeh():
     # Rendering
     #
     #
-
+    def cb_render_trajectories(self, attr, old, new):
+        self.render_trajectories()
     def render_trajectories(self):
         """
             This functions renders the trajectories plot. 
@@ -141,14 +195,30 @@ class automatumDataBokeh():
 
 
         data_dict = copy.deepcopy(self.static_world_line_dict )
+        point_dict = {
+            "xdata": [],
+            "ydata": [],
+            "color": [],
+            "info": []
+        }
         
         for color, obj in zip(colors, obj_list):
 
+            if(0 in self.trj_plot_options.active and obj.lane_change_flag_vec is not None):
+                for x, y, lane_change_flag, time in zip(obj.x_vec, obj.y_vec, obj.lane_change_flag_vec, obj.time):
+                    if(lane_change_flag):
+                        point_dict["xdata"].append(x)
+                        point_dict["ydata"].append(y)
+                        point_dict["color"].append("red")
+                        point_dict["info"].append("LC of %s at %.2f" % (obj.UUID, time))
+
             data_dict["xdata"].append(obj.x_vec)
             data_dict["ydata"].append(obj.y_vec)
             data_dict["color"].append(color)
+            data_dict["info"].append("UUID: %s" % obj.UUID)
 
         self.traj_plot_data.data = data_dict
+        self.lane_change_point_data.data = point_dict
     #
     #
     # Callbacks
@@ -167,10 +237,18 @@ class automatumDataBokeh():
         self.dataset = droneDataset(path2Dataset)
         #self.dataset.calculate_on_demand_values_for_all_objects()
 
+        self.uuid_color_mapping_dict = dict()
+        for obj, color in zip(self.dataset.dynWorld.get_list_of_dynamic_objects(), itertools.cycle(palette_major)):
+            self.uuid_color_mapping_dict[obj.UUID] = color
+            
+
         self.render_trajectories()
         self.render_live_plot(0)
         self.curDoc.add_periodic_callback(self.periodic_call_back, self.dataset.dynWorld.delta_t*1000)
 
+
+         
+
     def play_video(self):
         """
             Callback play button. 
@@ -259,14 +337,19 @@ class automatumDataBokeh():
         # Add Figures
         self.livePlotTitle = Title()
         self.livePlotTitle.text = 'Live plot of dataset:'
-        self.live_plot = Figure(plot_width=1000, plot_height=1000, match_aspect=True)
+        TOOLTIPS = [
+            ("Info", "@info")
+        ]           
+        self.live_plot = Figure(plot_width=1000, plot_height=1000, match_aspect=True, tooltips=TOOLTIPS)
         self.live_plot.title = self.livePlotTitle
         self.live_plot_data = ColumnDataSource(data={
             "xdata": [],
             "ydata": [],
             "color": [],
+            "info": []
 
         })
+     
         self.live_plot.multi_line(xs='xdata', ys='ydata', source=self.live_plot_data, line_color='color')
 
         self.start_button =  Button(label="Play", max_width = 150)
@@ -283,13 +366,18 @@ class automatumDataBokeh():
 
         self.animation_time_text_field = TextInput(value="0.0", title="Jump to time:", max_width = 200)
         self.go_button =  Button(label="Go", max_width = 200)
-        self.go_button.on_click(self.go_video)           
+        self.go_button.on_click(self.go_video)       
+
+        LABELS = ["Show Object Relations", "Show Distance to Lane Marking"]
+
+        self.extra_plot_option_checkbox = CheckboxGroup(labels=LABELS, active=[0, 1])            
 
         
         return row(column(self.start_button, 
                         self.pause_button, 
                         row(self.step_backwards_button, self.step_forwards_button), 
                         self.play_speed_slider,
+                        self.extra_plot_option_checkbox,
                         self.animation_time_text_field, 
                         self.go_button,
                          max_width = 250, min_width = 250 ), self.live_plot)
@@ -302,18 +390,35 @@ class automatumDataBokeh():
         # Add Figures
         t = Title()
         t.text = 'Trajectories of dataset:'
-        self.traj_plot = Figure(plot_width=1000, plot_height=1000, match_aspect=True)
+        TOOLTIPS = [
+            ("Info", "@info")
+        ]           
+        self.traj_plot = Figure(plot_width=1000, plot_height=1000, match_aspect=True, tooltips=TOOLTIPS)
         self.traj_plot.title = t
         self.traj_plot_data = ColumnDataSource(data={
             "xdata": [],
             "ydata": [],
             "color": [],
+            "info": []
         })
 
+        self.lane_change_point_data = ColumnDataSource(data={
+            "xdata": [],
+            "ydata": [],
+            "color": [],
+            "info": []
+        })        
+
+        LABELS = ["Show Lane Changes"]
+
+        self.trj_plot_options = CheckboxGroup(labels=LABELS, active=[0])            
+
         self.traj_plot.multi_line(xs='xdata', ys='ydata', source=self.traj_plot_data, line_color='color')
+        self.traj_plot.circle(x='xdata', y='ydata', color='color', source=self.lane_change_point_data)
+        self.trj_plot_options.on_change("active", self.cb_render_trajectories)
 
 
-        return row(column(max_width = 250, min_width = 250),self.traj_plot)
+        return row(column(self.trj_plot_options, max_width = 250, min_width = 250),self.traj_plot)
 
     def create_data_set_choice(self):
         """
