@@ -14,10 +14,11 @@ class dynamicWorld():
         :var frame_count:  Number of images/frames 
         :var fps:  Frame Rate 
         :var delta_t:  Time between two frames (1/fps) \[s\]
-        :var environmental:  Additional environmental info, see dataset documentation for further information.
         :var utm_referene_point: Reference point in the world coordinate system in UTM-Format. This reference point is the center of the coordinate system for the given position. The points is given as a tuple of (x \[m\], y \[m\], letter, number)-
         :var dynamicObjects: List of dynamic object. Its recommended to use the included functions to access the dynamic objects. 
         :var maxTime:  Maximum time of the dataset \[s\]
+        :var DrivenDistanceInMeter: Accumulated driven distance in meter
+        :var MedianDrivenDistanceInMeter: Median driven distance 
 
     """
     def __init__(self, path2JSON):
@@ -27,21 +28,29 @@ class dynamicWorld():
             :param dataSetFolderPath: Path to a folder containing a valid automatum.data dataset.
         """        
         if(not os.path.isfile(path2JSON)):
-            raise FileNotFoundError("The dynamic world json couldn't be found - Path: %s"%path2JSON)
+            raise FileNotFoundError("The dynamic world json couldn't be found - Path: %s" % path2JSON)
 
         with open(path2JSON) as f:
             dynamicWorldData = json.load(f) 
+        
+        if(dynamicWorldData["Release"] < 3.0):
+            raise Exception("This version is only compatible with the released datasets of version 3.x. If you still want to use version 1.x go to the compatibility table in the documentation and install the correct version. (https://openautomatumdronedata.readthedocs.io/en/latest/readme_include.html).")
 
         self.UUID = dynamicWorldData["UUID"]
 
         self.frame_count = dynamicWorldData["videoInfo"]["frame_count"]
         self.fps = dynamicWorldData["videoInfo"]["fps"]
-        self.delta_t = 1/self.fps
-        self.environmental = dynamicWorldData["EnvironmentInfo"]
+        self.delta_t = 1 / self.fps
         self.utm_referene_point = dynamicWorldData["UTM-ReferencePoint"]
-        self.dynamicObjects =  dict()
-        self.maxTime  = 0
+        self.dynamicObjects = dict()
+        self.maxTime = 0
+
+
+        self.DrivenDistanceInMeter = dynamicWorldData["DrivenDistanceInMeter"]
+        self.MedianDrivenDistanceInMeter = dynamicWorldData["MedianDrivenDistanceInMeter"]
+       
         for dynObjData in dynamicWorldData["objects"]:
+
             dynObj = dynamicObject.dynamic_object_factory(dynObjData, self.delta_t)
             self.dynamicObjects[dynObjData["UUID"]] = dynObj
             self.maxTime = max(self.maxTime, dynObj.get_last_time())
@@ -117,6 +126,15 @@ class dynamicObject():
         :var ax_vec:  Acceleration of the object in X-direction **in the vehicle coordinate system**
         :var ay_vec:  Acceleration of the object in Y-direction **in the vehicle coordinate system**
         :var time:  Vector of the timestamp in the dataset recording for the mention values. 
+        :var psi_vec:  Vector of orientation of objects 
+        :var lane_id_vec:  Id of the lane of the object in the corresponding xodr 
+        :var road_id_vec:  Id of the road of the object in the corresponding xodr 
+        :var object_relation_dict_list: Gives the UUID of all sourind objects as a dict in format ```{"front_ego": <uuid>,  "behind_ego": <uuid>,  "front_left": <uuid>,  "behind_left": <uuid>,  "front_right": <uuid>,  "behind_right": <uuid>}``` If an object relation does not exist, e.g. since there is no object, than the value will be ```None``. 
+        :var lane_change_flag_vec: Vector of bool values, whereby True means that a lane change occured
+        :var distance_left_lane_marking: Vector which gives the distance to the left lane marking
+        :var distance_right_lane_marking: Vector which gives the distance to the right lane marking
+        :var tth_dict_vec: Gives the Time-To-Collsion to all sourind objects as a dict in format ```{"front_ego": <ttc>,  "behind_ego": <ttc>,  "front_left": <ttc>,  "behind_left": <ttc>,  "front_right": <ttc>,  "behind_right": <ttc>}``` If the value can not be calculated, e.g. the relevant object is to fast, than the value will be set to ```-1```. If no object exists, the value will be set to ```None``` 
+        :var ttc_dict_vec: Gives the Time-To-Headway to all sourind objects as a dict in format ```{"front_ego": <tth>,  "behind_ego": <tth>,  "front_left": <tth>,  "behind_left": <tth>,  "front_right": <tth>,  "behind_right": <tth>}``` If the value can not be calculated, e.g. the relevant object is to fast, than the value will be set to ```-1```. If no object exists, the value will be set to ```None``` 
 
     """
     def __init__(self, movement_dynamic, delta_t):
@@ -131,11 +149,49 @@ class dynamicObject():
         self.width = movement_dynamic["width"]
         self.time = movement_dynamic["time"]
         self.UUID = movement_dynamic["UUID"]
+
         self.delta_t = delta_t
 
-        # On demand values_
-        self.lane_id = None 
-        self.object_relation_dict_list = None
+
+
+        # Values which are not part of all datasets 
+        if("lane_id_vec" in movement_dynamic):
+            self.lane_id_vec = movement_dynamic["lane_id_vec"]
+        else:
+            self.lane_id_vec = None            
+        if("road_id_vec" in movement_dynamic):            
+            self.road_id_vec = movement_dynamic["road_id_vec"]
+        else:
+            self.road_id_vec = None
+        if("object_relation_dict_list" in movement_dynamic):             
+            self.object_relation_dict_list = movement_dynamic["object_relation_dict_list"]
+        else:
+            self.object_relation_dict_list = None
+
+        if("lane_change_flag_vec" in movement_dynamic):
+            self.lane_change_flag_vec = movement_dynamic["lane_change_flag_vec"]
+        else:
+            self.lane_change_flag_vec = None 
+        if("distance_left_lane_marking" in movement_dynamic):
+            self.distance_left_lane_marking = movement_dynamic["distance_left_lane_marking"]
+        else:
+            self.distance_left_lane_marking = None 
+        if("distance_right_lane_marking" in movement_dynamic):
+            self.distance_right_lane_marking = movement_dynamic["distance_right_lane_marking"]
+        else:
+            self.distance_right_lane_marking = None 
+
+        if("tth_dict_vec" in movement_dynamic):
+            self.tth_dict_vec = movement_dynamic["tth_dict_vec"]
+        else:
+            self.tth_dict_vec = None 
+
+        if("ttc_dict_vec" in movement_dynamic):
+            self.ttc_dict_vec = movement_dynamic["ttc_dict_vec"]
+        else:
+            self.ttc_dict_vec = None                         
+            
+
 
     def __eq__(self, other):
         """Overwrite compare operator
@@ -152,37 +208,42 @@ class dynamicObject():
 
     def get_first_time(self):
         """
-            Returns the time the object occurs the first time
+            :return: Returns the time the object occurs the first time
         """
         return self.time[0]
 
     def get_last_time(self):
         """
-            Returns the time the object occurs the last time
+            :return: Returns the time the object occurs the last time
         """
         return self.time[-1]
 
     def is_visible_at(self, time):
         """
             Checks if the object is visible at the given time
+            :return: Returns true if the object is visiable.
         """
-        return (time > self.get_first_time() - self.delta_t/2 and time < self.get_last_time() + self.delta_t/2)
+        return (time > self.get_first_time() - self.delta_t / 2 and time < self.get_last_time() + self.delta_t / 2)
 
     def next_index_of_specific_time(self, time):
         """
             Returns the index that is next to the given time. 
             If the object is not visible in that time step. 
             The function returns None.
+
+            :return: Next index to a given time. 
         """
         if(not self.is_visible_at(time)):
             return None
-        return max(0,min(len(self),round((time - self.get_first_time())/self.delta_t)))
+        return max(0, min(len(self), round((time - self.get_first_time()) / self.delta_t)))
 
     def get_object_relation_for_defined_time(self, time):
         """Returns object relation for a given time stamp. 
-        Returns None if object relation are not calculated or object not in that time
+        Returns None if object relation are not calculated or object not in that time.
+
 
         :param time: Evaluted Time
+        :return: Dict of objects relations in the format ```{"front_ego": <uuid>,  "behind_ego": <uuid>,  "front_left": <uuid>,  "behind_left": <uuid>,  "front_right": <uuid>,  "behind_right": <uuid>}``` If an object relation does not exist, e.g. since there is no object, than the value will be ```None``. 
 
         """
         if(self.object_relation_dict_list is not None):
@@ -197,89 +258,15 @@ class dynamicObject():
         Returns None if object relation are not calculated or object not in that time
 
         :param time: Evaluted Time
-
+        :return: y position (None if object does not exist on the given time step)
+        :return: y position (None if object does not exist on the given time step)
         """
         if(self.object_relation_dict_list is not None):
             idx = self.next_index_of_specific_time(time)
             if(idx is not None):
                 return self.x_vec[idx], self.y_vec[idx]
-        return None
+        return None, None
 
-
-    def calculate_object_relations(self, surrounding_object_list, time):
-        """Calculations the relations between objects
-
-        :param surrounding_object_list: List of surrounding objects. (included the called one)
-        :param time: Time
-        :raises Exception: If the object releations are not calculated step by step
-        """
-
-        uuid_obj_in_front_ego = None
-        uuid_obj_in_behind_ego = None
-        uuid_obj_in_front_left = None
-        uuid_obj_in_behind_left = None
-        uuid_obj_in_front_right = None
-        uuid_obj_in_behind_right = None
-        ego_idx = self.next_index_of_specific_time(time)
-        ego_lane_id = self.lane_id[ego_idx]
-        obj_on_ego_lane = filter(lambda obj: obj.lane_id[obj.next_index_of_specific_time(time)] == ego_lane_id and obj.UUID != self.UUID, surrounding_object_list)
-        obj_on_right_lane = filter(lambda obj: obj.lane_id[obj.next_index_of_specific_time(time)] == ego_lane_id + (1 * np.sign(ego_lane_id)), surrounding_object_list)
-        obj_on_left_lane = filter(lambda obj: obj.lane_id[obj.next_index_of_specific_time(time)] == ego_lane_id + (-1 * np.sign(ego_lane_id)), surrounding_object_list)
-
-        x_vec_ego = dict()
-        for obj in obj_on_ego_lane:
-            x, y = obj.rotate_and_translate_position(-self.x_vec[ego_idx], -self.y_vec[ego_idx], -self.psi_vec[ego_idx], time)
-            x_vec_ego[obj.UUID] = x
-        x_vec_ego_pos = {k:v for k,v in x_vec_ego.items() if v > 0}
-        x_vec_ego_neg = {k:v for k,v in x_vec_ego.items() if v < 0}
-
-
-        if(len(x_vec_ego_pos) > 0):
-            uuid_obj_in_front_ego = min(x_vec_ego_pos, key=x_vec_ego_pos.get)
-        if(len(x_vec_ego_neg) > 0):
-            uuid_obj_in_behind_ego = max(x_vec_ego_neg, key=x_vec_ego_neg.get)
-
-
-        x_vec_right = dict()
-        for obj in obj_on_right_lane:
-            x, y = obj.rotate_and_translate_position(-self.x_vec[ego_idx], -self.y_vec[ego_idx], -self.psi_vec[ego_idx], time)
-            x_vec_right[obj.UUID] = x
-        x_vec_right_pos = {k:v for k,v in x_vec_right.items() if v > 0}
-        x_vec_right_neg = {k:v for k,v in x_vec_right.items() if v < 0}
-
-        if(len(x_vec_right_pos) > 0):
-            uuid_obj_in_front_right = min(x_vec_right_pos, key=x_vec_right_pos.get)
-        if(len(x_vec_right_neg) > 0):
-            uuid_obj_in_behind_right = max(x_vec_right_neg, key=x_vec_right_neg.get)
-
-
-        x_vec_left = dict()
-        for obj in obj_on_left_lane:
-            x, y = obj.rotate_and_translate_position(-self.x_vec[ego_idx], -self.y_vec[ego_idx], -self.psi_vec[ego_idx], time)
-            x_vec_left[obj.UUID] = x
-        x_vec_left_pos = {k:v for k,v in x_vec_left.items() if v > 0}
-        x_vec_left_neg = {k:v for k,v in x_vec_left.items() if v < 0}
-
-        if(len(x_vec_left_pos) > 0):
-            uuid_obj_in_front_left = min(x_vec_left_pos, key=x_vec_left_pos.get)
-        if(len(x_vec_left_neg) > 0):
-            uuid_obj_in_behind_left = max(x_vec_left_neg, key=x_vec_left_neg.get)
-
-        if(self.object_relation_dict_list is None):
-            self.object_relation_dict_list = list()
-
-        if(ego_idx == len(self.object_relation_dict_list)):
-            self.object_relation_dict_list.append({
-                "front_ego":uuid_obj_in_front_ego,
-                "behind_ego":uuid_obj_in_behind_ego,
-                "front_left":uuid_obj_in_front_left,
-                "behind_left":uuid_obj_in_behind_left,
-                "front_right":uuid_obj_in_front_right,
-                "behind_right":uuid_obj_in_behind_right
-            })
-
-        else:
-            raise Exception("calculate_object_relations have to be called timestep after timestep")
         
     @staticmethod
     def dynamic_object_factory(obj_data_dict, delta_t):
@@ -288,11 +275,11 @@ class dynamicObject():
             specified in the json right into the corresponding objects.
         """
         obj_factory_dict = {
-            "car":carObject,
-            "van":carObject,
-            "truck":truckObject,
-            "carWithTrailer":carWithTrailerObject,
-            "motorcycle":motorcycleObject
+            "car": carObject,
+            "van": vanObject,
+            "truck": truckObject,
+            "carWithTrailer": carWithTrailerObject,
+            "motorcycle": motorcycleObject
         }
         return obj_factory_dict[obj_data_dict["objType"]](obj_data_dict, delta_t)
 
@@ -354,6 +341,14 @@ class carWithTrailerObject(dynamicObject):
     """
     def __init__(self, movement_dynamic, delta_t):
         dynamicObject.__init__(self, movement_dynamic, delta_t)
+
+class vanObject(dynamicObject):
+    """
+        Class for representing a van object. 
+        Inheritances from dynamicObject which currently provides the main functionality. 
+    """
+    def __init__(self, movement_dynamic, delta_t):
+        dynamicObject.__init__(self, movement_dynamic, delta_t)
         
 
 class motorcycleObject(dynamicObject):
